@@ -1,6 +1,7 @@
 #       misc - miscellaneaous functions
 #
 # Copyright (c) 1993, 1994, 1995, 1996, 1997  The TERENA Association
+# Copyright (c) 1998                              RIPE NCC
 #
 # All Rights Reserved
 #
@@ -19,51 +20,31 @@
 # AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #
-# $Id: misc.pl,v 2.13 1997/10/21 15:00:35 chris Exp $
+# $Id: misc.pl,v 2.14.2.4 1998/11/25 10:34:05 roman Exp $
 #
 #	$RCSfile: misc.pl,v $
-#	$Revision: 2.13 $
-#	$Author: chris $
-#	$Date: 1997/10/21 15:00:35 $
+#	$Revision: 2.14.2.4 $
+#	$Author: roman $
+#	$Date: 1998/11/25 10:34:05 $
 #
 
 require "defines.pl";
+require "dpr.pl";
 
-# sort block for sorting changed fields
-# needs to cope with 6 or 8 digit dates (YYMMDD or YYYYMMDD)
-# regexp is tied with '$' so they will match 6 LSB digits and ignore
-# century part.
+# sort subroutine for sorting changed fields by date
 
 sub sortchangedfield {
-    
-    $a=~ /(\d{6,6})$/;
-    local($date1)=$1;
-    
-    $b=~ /(\d{6,6})$/;
-    
-    # print STDERR "-$a-$b-$date1-$1-\n";
-      
-    if ((!$date1) && (!$1)) {
-       
-       return 0;
-       
-    }
-    elsif (!$date1) {
-       
-       return -1;
-       
-    }
-    elsif (!$1) {
-       
-       return 1;
-       
-    }
-    else {
-       
-       return $date1 <=> $1;
-    
-    }
-    
+  $a =~ / ((?:\d{2}){3,4})$/;	# matches YYMMDD or YYYYMMDD
+  my($date1) = YYMMDDtoYYYYMMDD($1);
+
+  $b =~ / ((?:\d{2}){3,4})$/;	# matches YYMMDD or YYYYMMDD
+  my($date2) = YYMMDDtoYYYYMMDD($1);
+
+  return 0 unless $date1 || $date2;
+  return -1 unless $date1;
+  return 1 unless $date2;
+
+  return $date1 <=> $date2;
 }
 
 sub ReplaceGlobalVars {
@@ -94,7 +75,9 @@ sub ReplaceGlobalVars {
     $text=~ s/\$AUTOBOX/$AUTOBOX/go;
     $text=~ s/\$DATE/$DATE/go;
     $text=~ s/\$TIME/$TIME/go;
-    
+    $text=~ s/\$REFERHOST/$REFERHOST/go;
+    $text=~ s/\$REFERPORT/$REFERPORT/go;
+    $text=~ s/\$REFERMSG/$REFERMSG/go;
 }
 
 sub MakeRegular {
@@ -533,19 +516,41 @@ sub isnet {
 
 # the name says all ...:
 
-sub getYYMMDDandHHMMSS {
+sub getYYYYMMDDandHHMMSS {
 
     local($s,$m,$h,$day,$month,$year,$wd,$yd,$is)=localtime(time);
 
-    $YYMMDD=sprintf("%2d%2d%2d",$year,++$month,$day);
-    $YYMMDD=~ s/\D/0/g;
+    $YYYYMMDD=sprintf("%4d%2d%2d",$year + 1900,++$month,$day);
+    $YYYYMMDD=~ s/\D/0/g;
 
     $HHMMSS=sprintf("%2d:%2d:%2d",$h,$m,$s);
     $HHMMSS=~ s/[^\d:]/0/g;
 
-    print STDERR "$YYMMDD $HHMMSS $YEAR\n" if $DEBUG;                    
+    print STDERR "$YYYYMMDD $HHMMSS $YEAR\n" if $DEBUG;                    
 
-    return ($YYMMDD,$HHMMSS);
+    return ($YYYYMMDD,$HHMMSS);
+}
+
+#  YYMMDDtoYYYYMMDD
+#
+#  converts a date with six digits to y2k-compliant form with 8 digits
+#
+#  assumes that years 00-69 are 2000-2069 and 70-99 are 1970-1999 as the
+#  UNIX calendar starts at 1970 :-)
+#
+#  passes dates which are already in 8 digit form safely through
+
+sub YYMMDDtoYYYYMMDD {
+    my($origdate) = shift;
+
+    if (length($origdate) == 8) {
+      return $origdate;
+    }
+    if (length($origdate) == 6) {
+      return (substr($origdate, 0, 2) < 70 ? '20' : '19') . $origdate;
+    }
+    &syslog("ERRLOG", "YYMMDDtoYYYYMMDD: illegal date \"$origdate\"");
+    return undef;
 }
 
 # 
@@ -631,6 +636,8 @@ sub isnichandle {
     #    - country codes are allowed for APNIC compatibility
     #    - APNIC AP|CC-AU exceptions are handled correctly
     #    - -ORG organization InterNIC handles
+    #    - -ARIN ARIN handles
+    #    - -ORG-ARIN ARIN handles
     
   my $handle = shift;
   
@@ -654,6 +661,8 @@ sub isnichandle {
   # APNIC use two letter country code suffix
   # Austraila have used -1-AU, -2-AU, -CC-AU suffix.
   # Internic used -ORG suffix
+  # ARIN use -ARIN suffix
+  # ARIN also use -ORG-ARIN suffix
   #
   
   if ($handle =~ /^([A-Z]{2,$MAXLENGTHINITIALS})([1-9]\d{0,5})?(-\S+)?$/) {
@@ -681,6 +690,8 @@ sub isnichandle {
 		 ($suffix eq '1-AU') ||
 		 ($suffix eq '2-AU') ||
 		 ($suffix eq 'ORG') ||
+                 ($suffix eq 'ARIN') ||
+                 ($suffix eq 'ORG-ARIN') ||
 		 ($suffix eq 'AP')
 		 );
   }
@@ -942,7 +953,7 @@ sub isname {
 sub isphone {
     
     return scalar(($_[0] =~ /^\s*\+\s*\(?\s*\d+\s*\)?\s*(\s*[\-\.\(]*\s*\d+\s*\)?\s*)*(\s*ext\.\s*\d+\s*)?$/i) &&
-            (&isbrace($_[0])));
+            (&isparen($_[0])));
     
 }
 
@@ -970,13 +981,19 @@ sub isemail {
 
 }
 
-sub isbracket {
+# check for matching curly braces (was previously isbracket but
+# bracets are [] as far as I know :-) //snabb 980928
+
+sub isbrace {
     
     return ($_[0]=~ s/\{/\{/g==$_[0]=~ s/\}/\}/g);
     
 }
 
-sub isbrace {
+# check for matching parentheses (was previously isbrace but
+# braces are {}...) //snabb 980928
+
+sub isparen {
    
     return ($_[0]=~ s/\(/\(/g==$_[0]=~ s/\)/\)/g);
 
@@ -1028,10 +1045,10 @@ sub unlock {
     sub lower_priority {
 
       if (defined $RENICECMD) {
-	print STDERR "using [$RENICECMD]\n" if $opt_V;
+#	&dpr("using [$RENICECMD]\n");
 	return system("$RENICECMD 10 $$ > /dev/null 2>/dev/null");
       } else {
-	print STDERR "using setpriority(2)\n" if $opt_V;
+#	&dpr("using setpriority(2)\n");
 	return setpriority PRIO_PROCESS, 0, $nice_value;
       }
     }
