@@ -1,6 +1,7 @@
 # syntax.pl
 #
 # Copyright (c) 1993, 1994, 1995, 1996, 1997  The TERENA Association
+# Copyright (c) 1998                              RIPE NCC
 #
 # All Rights Reserved
 #
@@ -19,12 +20,12 @@
 # AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #
-# $Id: syntax.pl,v 2.8 1997/10/21 14:32:21 chris Exp $
+# $Id: syntax.pl,v 2.11.2.6 1998/11/25 10:34:10 roman Exp $
 #
 #	$RCSfile: syntax.pl,v $
-#	$Revision: 2.8 $
-#	$Author: chris $
-#	$Date: 1997/10/21 14:32:21 $
+#	$Revision: 2.11.2.6 $
+#	$Author: roman $
+#	$Date: 1998/11/25 10:34:10 $
 #
 #       ARGUMENTS:      *ASSOC object
 #       RETURNS:        INTEGER object_status
@@ -46,6 +47,7 @@ require "adderror.pl";
 require "net2net.pl";    
 require "misc.pl";	 
 require "handle.pl";
+require "dpr.pl";
 
 sub requiredsyntaxcheck {
     local(*object, $type)=@_;
@@ -90,14 +92,12 @@ sub requiredsyntaxcheck {
 
     if ($object{"uo"}) {
     
-       print STDERR "requiredsyntaxcheck - checking uo\n" if $opt_V;
-       
        if ($object{"uo"}=~ /^\S+.*\s+(\S+)$/) {
            
           if (crypt($1,$OVERRIDECRYPTEDPW) eq $OVERRIDECRYPTEDPW) {
               
              $object{"uo"}=~ s/\s+(\S+)$/ \*\*\*/;
-              
+#	     &dpr("valid override\n");
           }  
           else {
               
@@ -106,7 +106,7 @@ sub requiredsyntaxcheck {
              if (crypt($1,$OVERRIDECRYPTEDPW) eq $OVERRIDECRYPTEDPW) {
                 
                 $object{"uo"}=~ s/^(\S+)\s+/\*\*\* /;
-
+#		&dpr("valid override\n");
              }  
              else {
                  
@@ -152,8 +152,6 @@ sub checksyntax {
     local($rtcode) =$O_OK;
     local($itmp, $val, $msg);
 
-    print STDERR "checksyntax($rtcode) ($O_ERROR,$O_WARNING) - called\n" if $opt_V;
-
     foreach $itmp (keys %object) {
 	if ($object{$itmp} eq "") {
 	    ($val, $msg) = &dosyntax($itmp, "", *object);
@@ -296,7 +294,6 @@ sub checksyntax {
 		    $linewrap{"$peer:$wt"} = 1;
 		}
 		
-		print STDERR "Rebuilding \"$itmp\"\n" if $opt_V;
 		
 		if($FLAG eq "ai") {
 		  $object{$itmp} =~ s/from\s+|accept\s+//g;
@@ -369,8 +366,6 @@ sub checksyntax {
 	}
     }
     
-    print STDERR "checksyntax - returned\n" if $opt_V;
-    
     return $rtcode;
     
 }
@@ -379,7 +374,7 @@ sub dosyntax {
 
     local($key, $value, *object) = @_;
     
-    local($type)=&entype(*object);
+    local($type)=&entype(%object);
     
 
 #
@@ -397,8 +392,6 @@ sub dosyntax {
 #
     if ($key eq "ae") {
     
-       print STDERR "dosyntax - checking ae\n" if $opt_V;
-       
 	local($sugar1, $as, $sugar2, $rest) = "";
 	if($value =~ /^exclude/) {
 	    ($sugar1, $as, $sugar2, $rest) = split(/\s+/, $value, 4);
@@ -410,7 +403,7 @@ sub dosyntax {
 	}
 	if(!&isasnum($as)) {
 	    return $O_ERROR, 
-	    "syntax error in \"$ATTL{$key}: $value\" - neigbor peer".
+	    "syntax error in \"$ATTL{$key}: $value\" - neighbour peer".
 		" $as doesn't look like an AS";
 	}
 	if(&isasnum($rest) || 
@@ -430,8 +423,6 @@ sub dosyntax {
 #   
     if ($key eq "ai") {
     
-       print STDERR "dosyntax - checking ai\n" if $opt_V;
-       
 # 
 # This line has been pre-processed above.
 # remove syntax fluff, flip to unpper case for ases and remove leading WS
@@ -446,7 +437,7 @@ sub dosyntax {
 	local($as,$pref,$pol) = split(/\s+/,$value,3);
 	if (!&isasnum($as)) {
 	    return $O_ERROR, 
-	    "syntax error in \"$ATTL{$key}: $value\"\nneigbour peer".
+	    "syntax error in \"$ATTL{$key}: $value\"\nneighbour peer".
 		" $as doesn't look like an AS";
 	}
 	if (!$pref) {
@@ -464,18 +455,18 @@ sub dosyntax {
 		"routing policy expression given";
 	}
 #
-# now check equal brackets and braces
+# now check equal braces and parentheses
 #
-	if(!&isbracket($pol)) {
-	    return $O_ERROR, 
-	    "syntax error in \"$ATTL{$key}: peer $as cost $pref\"" .
-		"\n\tunequal brackets \"\(\)\"\n";
-	}
 	if(!&isbrace($pol)) {
 	    return $O_ERROR, 
-	    "syntax error in \"$ATTL{$key}: peer $as cost $pref\"" .
+	    "syntax error in \"$ATTL{$key}: $value\"" .
 		"\n\tunequal braces \"\{\}\"\n";
 	}
+	if(!&isparen($pol)) {
+	    return $O_ERROR, 
+	    "syntax error in \"$ATTL{$key}: $value\"" .
+		"\n\tunequal parentheses \"\(\)\"\n";
+	}
 #
 # Now grab the netlist entries and check they are ok
 #
@@ -483,19 +474,27 @@ sub dosyntax {
 	while($tmppol =~ s/(\{[^\}]*\})// ) {
 	    if(!&isnetlist($1)) {
 		return $O_ERROR, 
-		"syntax error in \"$ATTL{$key}: peer $as cost $pref\"".
+		"syntax error in \"$ATTL{$key}: $value\"".
 		    "\n\tnetlist error $1";
 	    }
 	}
 #
 # Now check the actual keywords
 #
-	while($tmppol =~ s/(\S+)//) {
-	    if (!&isaskeyword($1)) {
-		return $O_ERROR, 
-		"syntax error in \"$ATTL{$key}: peer $as cost $pref\"\n\t$1 ".
+# Joao. Including Curtis Villamizar's performance patch.
+#	while($tmppol =~ s/(\S+)//) {
+#	    if (!&isaskeyword($1)) {
+#		return $O_ERROR, 
+#		"syntax error in \"$ATTL{$key}: peer $as cost $pref\"\n\t$1 ".
+#
+#		    "is not a routing policy KEYWORD";
+        local($tmpword);
+        foreach $tmpword ( split(/\s+/, $tmppol) ) {
+ 	   if (!&isaskeyword($tmpword)) {
+ 	       return $O_ERROR, 
+ 	       "syntax error in \"$ATTL{$key}: $value\"\n".
+ 		   "\t$tmpword is not a routing policy KEYWORD";
 
-		    "is not a routing policy KEYWORD";
 	    }
 	}
 	return $O_OK;
@@ -505,8 +504,6 @@ sub dosyntax {
 #
     if ($key eq "al") {
     
-       print STDERR "dosyntax - checking al\n" if $opt_V;
-       
 	$value =~ tr/a-z/A-Z/;
 	local(@aslist) = split(/\s+/, $value);
 	local($i);
@@ -525,8 +522,6 @@ sub dosyntax {
 
     if ($key=~ /^an|la|or$/) {
     
-       print STDERR "dosyntax - checking an,la,or ($key)\n" if $opt_V;
-       
 	$value =~ tr/a-z/A-Z/;
 	
 	if (!&isasnum($value)) {
@@ -544,8 +539,6 @@ sub dosyntax {
 #
     if ($key eq "am") {
     
-       print STDERR "dosyntax - checking am\n" if $opt_V;
-       
 	if(!&isasmacro($value)) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\"";
 	}
@@ -556,8 +549,6 @@ sub dosyntax {
 #
     if ($key eq "ao") {
     
-       print STDERR "dosyntax - checking ao\n" if $opt_V;
-       
 	$value =~ s/to//;
 	$value =~ s/announce//;
 	$value =~ s/[aA][sS]/AS/g;
@@ -569,7 +560,7 @@ sub dosyntax {
 	if (!&isasnum($as)) {
 	    return $O_ERROR, 
 	    "syntax error in \"$ATTL{$key}: $value\" - ".
-		"neigbour peer $as doesn't look like an AS";
+		"neighbour peer $as doesn't look like an AS";
 	}
 	if (!$pol) {
 	    return $O_ERROR, 
@@ -577,17 +568,17 @@ sub dosyntax {
 		"no routing policy expression given";
 	}
 #
-# now check equal brackets and braces.
+# now check equal braces and parentheses
 #
-	if(!&isbracket($pol)) {
-	    return $O_ERROR, 
-	    "syntax error in \"$ATTL{$key}: peer $as \"" .
-		"\n\tunequal brackets \"\(\)\"\n";
-	}
 	if(!&isbrace($pol)) {
 	    return $O_ERROR, 
-	    "syntax error in \"$ATTL{$key}: peer $as \"" .
-		"\n\tunequal brackets \"\(\)\"\n";
+	    "syntax error in \"$ATTL{$key}: $value \"" .
+		"\n\tunequal braces \"\{\}\"\n";
+	}
+	if(!&isparen($pol)) {
+	    return $O_ERROR, 
+	    "syntax error in \"$ATTL{$key}: $value \"" .
+		"\n\tunequal parentheses \"\(\)\"\n";
 	}
 #
 # Now grab loop through netlist entries and check they are ok
@@ -597,14 +588,14 @@ sub dosyntax {
 	while($tmppol =~ s/(\{[^\}]*\})// ) {
 	    if(!&isnetlist($1)) {
 		return $O_ERROR, 
-		"syntax error in \"$ATTL{$key}: peer $as \"".
+		"syntax error in \"$ATTL{$key}: $value \"".
 		    "\n\tnetlist error $1";
 	    }
 	}
 	while($tmppol =~ s/(\S+)//) {
 	    if (!&isaskeyword($1)) {
 		return $O_ERROR, 
-		"syntax error in \"$ATTL{$key}: peer $as \"".
+		"syntax error in \"$ATTL{$key}: $value \"".
 		    "\n\t$1 is not a routing policy KEYWORD";
 	    }
 	}
@@ -618,8 +609,6 @@ sub dosyntax {
 #
     if ($key eq "at") {
     
-       print STDERR "dosyntax - checking at\n" if $opt_V;
-       
 	local(@authstr) = split(/\s+/, $value, 2);
 	if ($authstr[0] eq "NONE") {
 	    if ($authstr[1] !~ /^$/) {
@@ -653,6 +642,13 @@ sub dosyntax {
 	      return $O_OK;
 	    }
 	}
+        elsif ($authstr[0] =~ /^PGPKEY/) {
+            if ($authstr[0] !~ /^\s*PGPKEY-[0-9a-fA-F]{8}\s*$/) {
+               return $O_ERROR, "syntax error in \"$ATTL{$key}";
+            } else {
+                return $O_OK;
+            }
+        }
 	    
 	else {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\" $value";
@@ -664,8 +660,6 @@ sub dosyntax {
 #
     if ($key eq "au") {
     
-       print STDERR "dosyntax - checking au\n" if $opt_V;
-       
 	if ($value !~ /^[a-zA-Z0-9\-\.\ \,\(\)\/]+$/) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\"";
 	}
@@ -676,8 +670,6 @@ sub dosyntax {
 #
     if ($key eq "av") {
     
-       print STDERR "dosyntax - checking av\n" if $opt_V;
-       
 	local(@list) = split(/\s+/, $value);
         if (!&isasnum($list[0])) {
             return $O_ERROR, 
@@ -693,8 +685,6 @@ sub dosyntax {
 #
     if ($key eq "bg") {
     
-       print STDERR "dosyntax - checking bg\n" if $opt_V;
-       
 	if ($value !~ /^[A-Z0-9\-]+$/) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\"";
 	}
@@ -706,8 +696,6 @@ sub dosyntax {
 #
     if ($key eq "bi") {
     
-       print STDERR "dosyntax - checking bi\n" if $opt_V;
-       
 	local(@prefixes) = split(/\s+/, $value);
 	local($i);
 	if ($#prefixes > 1) {
@@ -726,14 +714,27 @@ sub dosyntax {
 #
     if ($key eq "bl") {
     
-       print STDERR "dosyntax - checking bl\n" if $opt_V;
-       
 	if ($value !~ /^[A-Z0-9\-\ ]+$/) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\"";
 	}
 	
 	return $O_OK;
     }
+
+#
+# ce - certif
+#
+    if ($key eq "ce") {
+                
+       print STDERR "dosyntax - checking ce\n" if $opt_V;
+            
+       unless ($value =~ /^[ -~]*$/) {
+           return $O_ERROR, "syntax error in \"$ATTL{$key}\", only printable ascii characters (0x20-0x7e) expected.";
+       }
+
+        return $O_OK;
+    }
+
 #
 # Try to do something clever with the changed field
 #
@@ -742,8 +743,6 @@ sub dosyntax {
 #
     if ($key=~ /^ch|wd$/) {
     
-       print STDERR "dosyntax - checking ch,wd ($key)\n" if $opt_V;
-       
        local($returncode)=$O_OK;
        
        local($date);
@@ -753,8 +752,10 @@ sub dosyntax {
        #
        # add current date if needed to last specified attribute
        
-       # match 6 or 8 digits YYMMDD or YYYYMMDD
-       if ($changedvalues[$#changedvalues]!~ / (\d{2}){3,4}$/) {
+       # match any string which looks like a date (we do the syntax
+       # check a little bit later)
+
+       if ($changedvalues[$#changedvalues]!~ / \d+$/) {
           
           $changedvalues[$#changedvalues].=" ".$DATE;
           
@@ -763,12 +764,12 @@ sub dosyntax {
           $returncode=$O_WARNING;
           
        }
-       
+
        #
        # sort values on date
        
        @changedvalues=sort sortchangedfield @changedvalues;
-       
+
        #print STDERR join("\n", "values:", @changedvalues);
        
        foreach $value (@changedvalues) {
@@ -776,7 +777,7 @@ sub dosyntax {
           $date="";
           
           if ($value=~ / /) {
-             $value=~ s/ +(\S+)$//;
+             $value=~ s/ +(\d+)$//;
              $date=$1;
           }
        
@@ -790,12 +791,17 @@ sub dosyntax {
 	     return $O_ERROR, "No date specified in \"$ATTL{$key}\" attribute";
           }
            
-	  if ($date=~ /^(\d{2}|\d{4})(\d\d)(\d\d)$/) {
+	  if ($date=~ /^(\d\d\d\d\d\d)$/) {  # match YYMMDD dates
+	     $date = YYMMDDtoYYYYMMDD($date);
+	     &addwarning(*object, "date in \"$ATTL{$key}\" ($1) changed to $date");
+	  }
+
+          # at this point all dates should be YYYYMMDD
+
+	  if ($date=~ /^(\d{4})(\d\d)(\d\d)$/) {
 	    
 	    local($year,$month,$day) = ($1,$2,$3);
 
-	    $year += 1900  if ($1<100);  # add century part if needed
-
 	    if ($year<1988) {
 	      return $O_ERROR,
 	        "date ($year in $date) part of \"$ATTL{$key}\" is older than the database itself! (1988)";
@@ -818,8 +824,7 @@ sub dosyntax {
 	
 	  }
 	  
-	  if (((length($date)==6) && ($date > $DATE)) ||
-	      ((length($date)==8) && ($date > ('19'.$DATE)))) {
+	  if ($date > $DATE) {
 	     
 	     &addwarning(*object, "date in \"$ATTL{$key}\" ($date) is in the future - changed to $DATE");
 	     $date=$DATE;
@@ -844,8 +849,6 @@ sub dosyntax {
 
     if ($key eq "cl") {
     
-       print STDERR "dosyntax - checking cl\n" if $opt_V;
-       
        foreach $j (split(/\s+/, $value)) {
      
           if (!&iscommunity($j)) {
@@ -865,8 +868,6 @@ sub dosyntax {
 
     if ($key eq "cm") {
     
-       print STDERR "dosyntax - checking cm\n" if $opt_V;
-       
 	if (!&iscommunity($value)) {
 	    return $O_ERROR, 
 	    "syntax error in \"$ATTL{$key}\" - ".
@@ -875,13 +876,34 @@ sub dosyntax {
 	return $O_OK;
     }
     
+# cn - cross-nfy
+#
+# value is a nic handle
+
+    if ($key eq 'cn') {
+      return $O_OK if isnichandle($value);
+
+      return ($O_ERROR,
+	      "syntax error in \"$ATTL{$key}\" - Not a nic handle ($value)");
+    }
+
+
+# ct - cross-mnt
+#
+# value is name of a mntner
+
+    if ($key eq 'ct') {
+      return $O_OK if ismaintainer($value);
+
+      return ($O_ERROR,
+	      "syntax error in \"$ATTL{$key}\" - Not a maintainer ($value)");
+    }
+
 #
 # cy - country
 
     if ($key eq "cy") {
     
-       print STDERR "dosyntax - checking cy\n" if $opt_V;
-       
        local($errors)="";
        local(@errors)=();
        
@@ -901,7 +923,7 @@ sub dosyntax {
        foreach $country (split(/\s+/, $value)) {
 	  
 	  if (!$COUNTRY{$country}) {
-	     push(@errors, $country)
+	      push(@errors, $country);
 	  }
 	  else {
 	  
@@ -969,7 +991,7 @@ sub dosyntax {
 	   }
 	   else {
 	      
-	      $errors='unknown country codes: '.join(', ',@errors).'"';
+	      $errors='unknown country codes: ' . join(', ',@errors) . "\"";
 	   
 	   }
 	   
@@ -1040,8 +1062,6 @@ sub dosyntax {
 
     if ($key=~ /^de|rm|tx|ad|tb$/) {
        
-       print STDERR "dosyntax - checking de,rm,tx,ad,tb ($key)\n" if $opt_V;
-    
        if ($value!~ /^.*$/) {
           return $O_ERROR, "invalid characters found in \"$ATTL{$key}\"";
        }
@@ -1056,8 +1076,6 @@ sub dosyntax {
 #
     if ($key eq "df") {
     
-       print STDERR "dosyntax - checking df\n" if $opt_V;
-       
 	local($rest) = "";
 	if ($object{"dp"}) {
 	    $value =~ tr/A-Z/a-z/;
@@ -1110,8 +1128,6 @@ sub dosyntax {
 #
     if ($key eq "di") {
     
-       print STDERR "dosyntax - checking di\n" if $opt_V;
-       
 	local(@list) = split(/\s+/,$value);
 	local($j) = 0;
 	foreach $j (0..$#list) {
@@ -1127,8 +1143,6 @@ sub dosyntax {
 #
     if ($key eq "dm") {
        
-       print STDERR "dosyntax - checking dm\n" if $opt_V;
-       
 	local($bis,$pref,@crap) = split(/\s+/,$value);
         if (!&isclnsprefix($bis)) {
             return $O_ERROR, 
@@ -1166,8 +1180,6 @@ sub dosyntax {
 
     if ($key=~ /^d[an]|ir$/) {
     
-       print STDERR "dosyntax - checking dn, da ($key)\n" if $opt_V;
-       
        if (!&isdomname($value)) {
           return $O_ERROR, "illegal domain name in ".$ATTL{$key};
        }
@@ -1176,13 +1188,39 @@ sub dosyntax {
 	
     }
     
+#
+# rf - refer
+# referral attribute for domain objects
+
+    if ($key eq "rf") {
+
+	local($type,$host,$port) = split(/\s+/,$value);
+        # Check syntax of TYPE field
+	if ($type !~ /^ripe|internic|simple$/i) {
+	    return $O_ERROR,
+	    "syntax error in \"$ATTL{$key}: $value\" - ".
+		"type must be one of RIPE, INTERNIC or SIMPLE";
+	}
+
+        # Check syntax of host field
+	if (!&isdomname($host)) {
+	    return $O_ERROR, "illegal host name in ".$ATTL{$key};
+	}
+
+        # Check syntax of port field
+	if (defined $port && ($port !~ /[0-9]{1,5}/ || $port > 65534) ) {
+	    return $O_ERROR, 
+	    "syntax error in \"$ATTL{$key}: $value\" - ".
+		"port must be numeric value in the range 0-65534";
+	}
+	return $O_OK;
+    }
+    
 #
 # do - dom-out
 #
     if ($key eq "do") {
     
-       print STDERR "dosyntax - checking do\n" if $opt_V;
-       
 	local($bis,@crap) = split(/\s+/,$value);
         if (!&isclnsprefix($bis)) {
             return $O_ERROR, 
@@ -1207,8 +1245,6 @@ sub dosyntax {
 #
     if ($key eq "dp") {
     
-       print STDERR "dosyntax - checking dp\n" if $opt_V;
-       
 	if (!&isclnsprefix($value)) {
 	    return $O_ERROR, "illegal NSAP prefix format in \"$ATTL{$key}\"";
 	}
@@ -1225,8 +1261,6 @@ sub dosyntax {
 
     if ($key=~ /^dt|em|gd|mn|ny|op$/) {
     
-       print STDERR "dosyntax - checking dt,em,gd,mn,ny,op ($key)\n" if $opt_V;
-       
 	if (!&isemail($value)) {
 	    return $O_ERROR, 
 	    "syntax error in \"$ATTL{$key}\" ".
@@ -1251,8 +1285,6 @@ sub dosyntax {
 #
     if ($key eq "ho") {
     
-       print STDERR "dosyntax - checking ho\n" if $opt_V;
-       
 	local($stat, $msg, @str) = &netpre_verify($value);
 	if($stat == $NOK) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\"\n$msg\n";
@@ -1264,8 +1296,6 @@ sub dosyntax {
 #
     if ($key eq "if") {
     
-       print STDERR "dosyntax - checking if\n" if $opt_V;
-       
 	local($if, $mask) = split(/\s+/, $value, 2);
 	if(!&isipaddr($if)) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\""
@@ -1283,8 +1313,6 @@ sub dosyntax {
 
     if ($key eq "ii") {
     
-       print STDERR "dosyntax - checking ii\n" if $opt_V;
-       
 	local(@iistr) = split(/\s+/,$value);	
 	if ($#iistr != 1 ) {
 	    return $O_ERROR, 
@@ -1307,8 +1335,6 @@ sub dosyntax {
 #
     if ($key eq "it") {
     
-       print STDERR "dosyntax - checking it\n" if $opt_V;
-       
 #
 # This line has been pre-processed above.
 # remove syntax fluff, flip to unpper case for ases and remove leading WS
@@ -1323,7 +1349,7 @@ sub dosyntax {
 	local($as, $lid, $rid, $pref, $pol) = split(/\s+/,$value, 5);
 	if (!&isasnum($as)) {
             return $O_ERROR,
-            "syntax error in \"$ATTL{$key}: $value\"\n\tneigbour peer".
+            "syntax error in \"$ATTL{$key}: $value\"\n\tneighbour peer".
                 " $as doesn't look like an AS";
         }
 	if (!&isipaddr($lid) || !&isipaddr($rid)) {
@@ -1349,18 +1375,18 @@ sub dosyntax {
                 "routing policy expression given";
         }
 #
-# now check equal brackets and braces
+# now check equal braces and parentheses
 #
-        if(!&isbracket($pol)) {
-            return $O_ERROR,
-            "syntax error in \"$ATTL{$key}: peer $as cost $pref\"" .
-                "\n\tunequal brackets \"\(\)\"\n";
-        }
         if(!&isbrace($pol)) {
             return $O_ERROR,
-            "syntax error in \"$ATTL{$key}: peer $as cost $pref\"" .
+            "syntax error in \"$ATTL{$key}: $value\"" .
                 "\n\tunequal braces \"\{\}\"\n";
         }
+        if(!&isparen($pol)) {
+            return $O_ERROR,
+            "syntax error in \"$ATTL{$key}: $value\"" .
+                "\n\tunequal parentheses \"\(\)\"\n";
+        }
 #
 # Now grab the netlist entries and check they are ok
 #
@@ -1368,7 +1394,7 @@ sub dosyntax {
         while($tmppol =~ s/(\{[^\}]*\})// ) {
             if(!&isnetlist($1)) {
                 return $O_ERROR,
-                "syntax error in \"$ATTL{$key}: peer $as cost $pref\"".
+                "syntax error in \"$ATTL{$key}: $value\"".
                     "\n\tnetlist error $1";
             }
         }
@@ -1378,7 +1404,7 @@ sub dosyntax {
         while($tmppol =~ s/(\S+)//) {
             if (!&isaskeyword($1)) {
                 return $O_ERROR,
-                "syntax error in \"$ATTL{$key}: peer $as cost $pref\"\n\t$1 ".
+                "syntax error in \"$ATTL{$key}: $value\"\n\t$1 ".
                     "is not a routing policy KEYWORD";
             }
         }
@@ -1389,8 +1415,6 @@ sub dosyntax {
 #
     if ($key eq "io") {
     
-       print STDERR "dosyntax - checking io\n" if $opt_V;
-       
 	local($gotmet) = 0;
 	local($as, $rid, $metric, $pol);
         $value =~ s/to//;
@@ -1409,7 +1433,7 @@ sub dosyntax {
         if (!&isasnum($as)) {
             return $O_ERROR,
             "syntax error in \"$ATTL{$key}: $value\" - ".
-                "neigbour peer $as doesn't look like an AS";
+                "neighbour peer $as doesn't look like an AS";
         }
         if (!&isipaddr($lid) || !&isipaddr($rid)) {
             return $O_ERROR,
@@ -1433,17 +1457,17 @@ sub dosyntax {
                 "no routing policy expression given";
         }
 #
-# now check equal brackets and braces
+# now check equal braces and parentheses
 #
-        if(!&isbracket($pol)) {
-            return $O_ERROR,
-            "syntax error in \"$ATTL{$key}: peer $as \"" .
-                "\n\tunequal brackets \"\(\)\"\n";
-        }
         if(!&isbrace($pol)) {
             return $O_ERROR,
-            "syntax error in \"$ATTL{$key}: peer $as \"" .
-                "\n\tunequal brackets \"\(\)\"\n";
+            "syntax error in \"$ATTL{$key}: $value \"" .
+                "\n\tunequal braces \"\{\}\"\n";
+        }
+        if(!&isparen($pol)) {
+            return $O_ERROR,
+            "syntax error in \"$ATTL{$key}: $value \"" .
+                "\n\tunequal parentheses \"\(\)\"\n";
         }
 #
 # Now grab the netlist entries and check they are ok
@@ -1452,14 +1476,14 @@ sub dosyntax {
         while($tmppol =~ s/(\{[^\}]*\})// ) {
             if(!&isnetlist($1)) {
                 return $O_ERROR,
-                "syntax error in \"$ATTL{$key}: peer $as \"".
+                "syntax error in \"$ATTL{$key}: $value \"".
                     "\n\tnetlist error $1";
             }
         }
         while($tmppol =~ s/(\S+)//) {
             if (!&isaskeyword($1)) {
                 return $O_ERROR,
-                "syntax error in \"$ATTL{$key}: peer $as \"".
+                "syntax error in \"$ATTL{$key}: $value \"".
                     "\n\t$1 is not a routing policy KEYWORD";
             }
         }
@@ -1472,13 +1496,11 @@ sub dosyntax {
 
     if ($key eq "in") {
 
-       print STDERR "dosyntax - checking in\n" if $opt_V;
-       
        $value=~ s/\s+/ /g;
        
        local($newvalue, $code)=&normalizerange($value, $key);
        
-       print STDERR "dosyntax - newrange: $newvalue code: $code (OK=$O_OK)\n" if ($opt_V);
+#       &dpr("dosyntax - newrange: $newvalue code: $code (OK=$O_OK)\n");
        
        if ($code==$O_OK) {
           
@@ -1510,8 +1532,6 @@ sub dosyntax {
           
           }
           
-          print STDERR "dosyntax - warnings: $warnings\n" if (($warnings) && ($opt_V));
-          
           return ($O_WARNING, $warnings) if ($warnings);
           
           return $O_OK; 
@@ -1551,8 +1571,6 @@ sub dosyntax {
 
     if ($key eq "i6") {
 
-       print STDERR "dosyntax - checking i6\n" if $opt_V;
-       
        $value=&fullipv6formatprefix($value);
        
        # print STDERR $value, "\n";
@@ -1618,13 +1636,26 @@ sub dosyntax {
 
 
     
+#
+# kc - key-cert
+#
+    if ($key eq "kc") {
+
+       print STDERR "dosyntax - checking kc\n" if $opt_V;
+
+       if ($value =~ m/^\s*PGPKEY-[0-9a-fA-F]{8}\s*$/i) {
+        return $O_OK;
+       } else {
+        return $O_ERROR, "syntax error in  \"$ATTL{$key}\", PGPKEY-hhhhhhhh expected (h = hexadecimal #)";
+       }
+    }
+
+
 #
 # li - limerick
 #
     if ($key eq "li") {
     
-       print STDERR "dosyntax - checking li\n" if $opt_V;
-    
        $value=~ tr/a-z/A-Z/;
        
        if (!&islimerick($value)) {
@@ -1645,8 +1676,6 @@ sub dosyntax {
 
     if ($key eq "lo") {
        
-       print STDERR "dosyntax - checking lo\n" if $opt_V;
-       
 	if ($value !~ /^[a-zA-Z0-9\-\.\ \,\(\)\&\'\"\/]+$/) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\"";
 	}
@@ -1660,8 +1689,6 @@ sub dosyntax {
 #
     if ($key eq "lr") {
     
-       print STDERR "dosyntax - checking lr\n" if $opt_V;
-       
 	local(@list) = split(/\s+/, $value);
 	if (!&isasnum($list[0])) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\" - $list[0] is not a valid peer";
@@ -1679,8 +1706,6 @@ sub dosyntax {
 #
     if ($key eq "ma") {
     
-       print STDERR "dosyntax - checking ma\n" if $opt_V;
-       
 	if ($value !~ /^[A-Z0-9\-]+$/) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\"";
 	}    
@@ -1693,8 +1718,6 @@ sub dosyntax {
 
     if ($key=~ /^m[bl]$/) {
 
-       print STDERR "dosyntax - checking mb,ml\n" if $opt_V;
-
        local($mntner);
 
        local(@syntaxerrors)=();
@@ -1709,7 +1732,7 @@ sub dosyntax {
 
        $value=~ tr/[a-z]/[A-Z]/;
 
-       foreach $mntner (split(/\s+/,$value)) {
+       foreach $mntner (split(/\n+/,$value)) {
 	  
 	  # check syntax first
 	  
@@ -1717,7 +1740,7 @@ sub dosyntax {
 	  
 	  # print STDERR "*$mntner*\n";
 	  
-	  if ($mntner!~ /^[A-Z0-9\-]+$/) {
+	  if (!&ismaintainer($mntner)) {
 	     push(@syntaxerrors, $mntner);
              next;
           }
@@ -1761,13 +1784,27 @@ sub dosyntax {
 	   return $O_ERROR, $errormsg;
 	}
 	else {   
-	   
 	   $object{$key}=join("\n", @mntners);
-	   
-	   return $O_OK;
-	   
 	}
-	
+	if ($key eq 'mb' && $type eq 'kc') {
+	   my($warningmsg) = "";
+
+           foreach $mntner (split(/\s+/,$value)) {
+	      my(%mnt);
+
+	      if (%mnt = GetMaintainer($mntner, $object{'so'}, 0)) {
+	         foreach (split(/\n/, $mnt{"at"})) {
+		    unless (/^PGPKEY/) {
+		       $warningmsg .= "maintainer $mntner does not provide "
+			   . "adequate protection for\nPGP key certificate "
+			   . "object -- consider upgrading auth to PGPKEY\n";
+		    }
+	         }
+	      }
+	   }
+           return ($O_WARNING, $warningmsg) if $warningmsg;
+	}
+	return $O_OK;
     }
     
 #
@@ -1775,8 +1812,6 @@ sub dosyntax {
 #
     if ($key eq "mt") {
        
-       print STDERR "dosyntax - checking mt\n" if $opt_V; 
-       
        # convert to uppercase
 
        $value=~ tr/[a-z]/[A-Z]/;
@@ -1796,8 +1831,6 @@ sub dosyntax {
 
     if ($key=~ /^[na]a$/) {
     
-        print STDERR "dosyntax - checking na,aa ($key)\n" if $opt_V; 
-        
 	local($cur) = $value;
 	local($changed) = 1 if $object{$key} =~ tr/a-z/A-Z/;
 	local($changed) = 1 if $object{$key} =~ tr/\./\-/;
@@ -1819,76 +1852,58 @@ sub dosyntax {
 # nh - nic-hdl
 
     if ($key eq "nh") {
-    
-       print STDERR "dosyntax - checking nh ($key)\n" if $opt_V; 
-       
-       $object{$key}=$value if ($value=~ tr/a-z/A-Z/);
-       
-       if ($value=~ /^${AUTONICPREFIXREGULAR}0*([1-9]\d*)?(\d)([A-Z]*)$/) {
-          
-          local($prefix)=$AUTONICPREFIX.$1.$2;
-          local($initials)=$3;
-          local($autoinitials)=0;
-          
-          #
+#       &dpr("dosyntax - checking nh ($key)\n");
+       $object{$key} = $value if ($value =~ tr/a-z/A-Z/);
+
+       if ($value =~ /^${AUTONICPREFIXREGULAR}0*([1-9]\d*)?(\d)([A-Z]*)$/) {
+          local($prefix) = $AUTONICPREFIX.$1.$2;
+          local($initials) = $3;
+          local($autoinitials) = 0;
+
           # check if the ID is not already used
-          
-          return $O_ERROR, "error in \"$ATTL{$key}\", auto NIC handle assignment\nidentification code ($1.$2) already used in other object" if ($NICID{$object{"so"}, $prefix});
-          
-          #
+          return $O_ERROR, "error in \"$ATTL{$key}\", auto NIC handle " . 
+	                   "assignment\nidentification code ($1.$2) " . 
+			   "already used in other object" 
+			       if ($NICID{$object{"so"}, $prefix});
+
           # find initials if not already specified
-          
           if (!$initials) {
-          
-             $autoinitials=1;
-             $initials=&findinitials($object{$type});
-          
+             $autoinitials = 1;
+             $initials = &findinitials($object{$type});
           }
           
           if (&isnichandle($initials."1\-".$NICPOSTFIX{$object{"so"}})) {
-             
-             $object{$key}=$prefix.$initials;
-             
-             #
+             $object{$key} = $prefix.$initials;
              # check if the ID is not already used
-             
-             $prefix=~ /\-(\d+)$/;
-             
-             return $O_ERROR, "error in \"$ATTL{$key}\", auto NIC handle assignment identification code ($1) already used in other object\nplease use another number then $1" if ($NICID{$object{"so"}, $prefix});
-          
+             $prefix =~ /\-(\d+)$/;
+             return $O_ERROR, "error in \"$ATTL{$key}\", auto NIC handle " .
+		              "assignment identification code ($1) already " .
+			      "used in other object\nplease use another " .
+			      "number than $1" 
+				  if ($NICID{$object{"so"}, $prefix});
              $NICID{$object{"so"}, $prefix}=1;
-             
              return $O_OK;
-             
           }
           else {
-             
              if ((length($initials)>$MAXLENGTHINITIALS) ||
                  (length($initials)<2)) {
-                 
                 if ($autoinitials) {
-                
-                   return $O_ERROR, "couldn\'t find a valid set of initials for NIC handle, please specify yourself: AUTO-#[Initials]"; 
-                
+                   return $O_ERROR, "couldn\'t find a valid set of initials" .
+		                    " for NIC handle, please specify " .
+				    "yourself: AUTO-#[Initials]"; 
                 }
                 else {
-                
-                   return $O_ERROR, "syntax error in \"$ATTL{$key}\", number of initials must be >=2 and <=$MAXLENGTHINITIALS"; 
-                
+                   return $O_ERROR, "syntax error in \"$ATTL{$key}\", " .
+		                    " number of initials must be >=2 and " .
+				    "<=$MAXLENGTHINITIALS"; 
                 }
-             
              }
-                              
              return $O_ERROR, "syntax error in \"$ATTL{$key}\"";
-             
           }
-          
        }
-       
-       return $O_ERROR, "syntax error in \"$ATTL{$key}\"" if (!&isnichandle($value));
-       
+       return $O_ERROR, "syntax error in \"$ATTL{$key}\"" 
+	   if (!&isnichandle($value));
        return $O_OK;
-       
     }
     
 #
@@ -1896,8 +1911,6 @@ sub dosyntax {
 
     if ($key eq "ni") {
     
-       print STDERR "dosyntax - checking ni ($key)\n" if $opt_V; 
-       
 	if ($value !~ /^[(\d=\d+)\s*]+$/) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\"";
 	}
@@ -1908,8 +1921,6 @@ sub dosyntax {
 #
     if ($key eq "no") {
     
-       print STDERR "dosyntax - checking no ($key)\n" if $opt_V; 
-       
 	if ($value !~ /^[(\d=\d+)\s*]+$/) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\"";
 	}
@@ -1920,8 +1931,6 @@ sub dosyntax {
 # 
     if ($key eq "ns") {
     
-       print STDERR "dosyntax - checking ns ($key)\n" if $opt_V; 
-       
 	@list = ();
 	@list = split(/\s+/,$value);
 	$j = 0;
@@ -1947,8 +1956,6 @@ sub dosyntax {
 
     if ($key=~ /^o[pf]|ph|fx$/) {
        
-       print STDERR "dosyntax - checking op,of,ph,fx ($key)\n" if $opt_V; 
-              
 	if (!(&isphone($value))) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\",\nphone number is not formatted as an international phone number";
 	}
@@ -1964,8 +1971,6 @@ sub dosyntax {
 
     if ($key eq "uo") {
 
-       print STDERR "dosyntax - checking uo ($key)\n" if $opt_V; 
-
        return $O_OK;
 
     }
@@ -1976,8 +1981,6 @@ sub dosyntax {
 
     if ($key eq "pe") {
        
-        print STDERR "dosyntax - checking pe ($key)\n" if $opt_V; 
-        
 	local(@peer) = split(/\s+/, $value);
 	if($value =~ /localas/) {
 	    if($peer[3] ne "localas" || !&isasnum($peer[4])) {
@@ -1997,101 +2000,105 @@ sub dosyntax {
 # pn - person
 # ro - role
 #
+    if ($key =~ /^pn|ro$/) {
+
+#	&dpr("checking pn,ro ($key)\n");
+	
+	my($nichdl) = $value;
+	my($nrofcomponents);
+	my($name) = $value;
+
+	# person and role attibutes are single so no need to split on \n
+
+	$nichdl=~ tr/a-z/A-Z/;
+
+	if ($nichdl=~ /^$AUTONICPREFIXREGULAR\d+[A-Z]*$/o) {
+
+	    return $O_ERROR, 
+	        "error in \"$ATTL{$key}\", " .
+		"$nichdl looks like auto NIC handle assignment reference\n" .
+		"instead of a $ATTL{$key} name";              
+        }
+	elsif ( !(($NROFNAMES < 2) &&
+		  ($nichdl =~/^[A-Z]+$/)) &&
+	       (&isnichandle($nichdl)) ) {
+	    
+	    return $O_ERROR,
+	        "error in \"$ATTL{$key}\", " .
+		"$nichdl looks like a NIC handle\n" .
+		    "instead of a $ATTL{$key} name" if ($nichandle =~ /\d/);
+	}
+	$name =~ s/ +/ /g;
+	return $O_ERROR, 
+	    "syntax error in \"$ATTL{$key}\" ($name) \n" .
+	    "use non-abbreviated first and last names"
+		if (($name =~ /^$TITLES/i) || 
+		    ($name =~ /^\S+\. /) || 
+		    ($name =~ /\S\.$/));
+
+        $nrofcomponents=0;
+       
+        foreach (split(/ /, $name)) {
+            return $O_ERROR, "syntax error in \"$ATTL{$key}\"" 
+		if (!&isname($_));
+            $nrofcomponents++ if ($name =~ /\S\S/);
+        }
+
+        return $O_ERROR, 
+	    "syntax error in \"$ATTL{$key}\" - " . 
+	    "name must contain at least $NROFNAMES components of at " .
+	    "least two characters" 
+		if ($nrofcomponents<$NROFNAMES);
+
+        $object{$key} = $name;
+
+	return $O_OK;
+    }
+
 # ac - admin-c
 # ah - author
 # tc - tech-c
 # zc - zone-c
 
-    if ($key=~ /^[atz]c|ah|pn|ro$/) {
+# new syntax checks - value of the above attributes is now enforced to 
+# contain a nic handle (RK - 19980917)
+
+    if ($key=~ /^[atz]c|ah$/) {
        
-       # print STDERR "before: ", $object{$key}, "\n";
-    
-       print STDERR "dosyntax - checking ac,ah,tc,zc,ro,pn ($key)\n" if $opt_V;
-       
-       local(@names)=();
-       
-       local($nichandle,$name,$nrofcomponents);
+       my(@names)=();
+       my($name);
        
        foreach $name (split(/\n+/, $value)) {
        
-          #
-          # person/role names should not contain a NIC handle
-          # but the other attributes preferably should ...
-       
-          local($nichandle)=$name;
-          $nichandle=~ tr/a-z/A-Z/;
+          my($nichdl) = $name;
+          $nichdl =~ tr/a-z/A-Z/;
           
-          # print STDERR "$nichandle*$AUTONICPREFIXREGULAR*$AUTONICPREFIXREGULAR\n";
-          
-          #
-          # auto NIC handle found
-          
-          if ($nichandle=~ /^$AUTONICPREFIXREGULAR\d+[A-Z]*$/o) {
-             
-             # print STDERR "auto nic found: $nichandle\n";
-             
-             if ($key=~ /^pn|ro$/) {
-                return $O_ERROR, "error in \"$ATTL{$key}\", name looks like auto NIC handle assignment reference\ninstead of a ".$ATTL{$key}." name";              
-             }
-             
-             push(@names, $nichandle);
-             
-             next;
-             
+          if ($nichdl =~ /^$AUTONICPREFIXREGULAR\d+[A-Z]*$/o) {
+#	      &dpr("Auto nic handle $nichdl referenced in \"$ATTL{$key}\"\n");
+              push(@names, $nichdl);
+              next;
           }
-          elsif ((!(($NROFNAMES<2) && ($nichandle=~ /^[A-Z]+$/))) &&
-                 (&isnichandle($nichandle))) {
-          
-             if ($key=~ /^pn|ro$/) {
-          
-                return $O_ERROR, "error in \"$ATTL{$key}\", name looks like a NIC handle\ninstead of a ".$ATTL{$key}." name" if ($nichandle=~ /\d/);
-       
-             }
-             else {
-               
-                push(@names, $nichandle);
-                
-                next;
-          
-             }
-             
+          elsif ( &isnichandle($nichdl) ) {
+              push(@names, $nichdl);
+              next;
           }
-       
-          $name=~ s/ +/ /g;
-          
-          return $O_ERROR, "syntax error in \"$ATTL{$key}\" - use non-abbreviated first & last names" if (($name=~ /^$TITLES/i) || ($name=~ /^\S+\. /) || ($name=~ /\S\.$/));
-       
-          $nrofcomponents=0;
-       
-          foreach (split(/ /,$name)) {
-          
-             return $O_ERROR, "syntax error in \"$ATTL{$key}\"" if (!&isname($_));
-             
-             $nrofcomponents++ if ($name=~ /\S\S/);
-             
-          }
-          
-          return $O_ERROR, "syntax error in \"$ATTL{$key}\" - name must contain at least $NROFNAMES components of at least two characters" if ($nrofcomponents<$NROFNAMES);
-       
-          push(@names, $name);
-          
+	  else {
+	      return $O_ERROR,
+	          "syntax error in \"$ATTL{$key}\"\n" .
+		  "$name is nor nic handle neither auto nic handle";
+	  }
        }
-       
        $object{$key}=join("\n", @names);
-       
-       # print STDERR $PASSWORD, "after: ", $object{$key}, "\n";
-       
        return $O_OK;
-    
     }
+
+
     
 #
 # rl - routpr-l
 
     if ($key eq "rl") {
     
-       print STDERR "dosyntax - checking rl ($key)\n" if $opt_V; 
-        
 	if ($value !~ /^[A-Z0-9\-\ ]+$/) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\"";
 	}
@@ -2103,8 +2110,6 @@ sub dosyntax {
 #
     if ($key eq "rp") {
     
-       print STDERR "dosyntax - checking rp ($key)\n" if $opt_V; 
-       
 	if ($value !~ /^[A-Z0-9\-]+$/) {
 	    return $O_ERROR, "syntax error in \"$ATTL{$key}\"";
 	}
@@ -2117,13 +2122,11 @@ sub dosyntax {
 #
     if ($key eq "rt") {
     
-       print STDERR "dosyntax - checking rt ($key)\n" if $opt_V; 
-       
        if (($value=~ /^[\d\.]+$/) || ($value=~ /\-/)) {
           
           local($newvalue, $code)=&normalizerange($value, "in");
        
-          print STDERR "dosyntax - newrange: $newvalue code: $code (OK=$O_OK)\n" if ($opt_V);
+#          &dpr("dosyntax - newrange: $newvalue code: $code (OK=$O_OK)\n");
        
           if ($code==$O_OK) {
              
@@ -2182,8 +2185,6 @@ sub dosyntax {
 #
     if ($key eq "rz") {
     
-       print STDERR "dosyntax - checking rz ($key)\n" if $opt_V; 
-       
 	@list = ();
 	@list = split(/\s+/,$value);
 	$j = 0;
@@ -2199,8 +2200,6 @@ sub dosyntax {
 #
     if ($key eq "sd") {
     
-       print STDERR "dosyntax - checking sd ($key)\n" if $opt_V; 
-       
 	foreach (split(/\s+/, $value)) {
 	    if (!&issubdomname($_)) {
 		return $O_ERROR, "syntax error in \"$ATTL{$key}\": $_";
@@ -2216,8 +2215,6 @@ sub dosyntax {
 
     if ($key eq "so") {
 
-       print STDERR "dosyntax - checking so ($key)\n" if $opt_V; 
-
        return $O_OK;
 
     }
@@ -2227,8 +2224,6 @@ sub dosyntax {
 #
     if ($key eq "st") {
     
-       print STDERR "dosyntax - checking st ($key)\n" if $opt_V; 
-       
        $value=~ tr/a-z/A-Z/;
        $value=~ s/\s+/ /g;
        $object{$key}=$value;  
@@ -2259,8 +2254,6 @@ sub dosyntax {
 #
     if ($key eq "tr") {
        
-       print STDERR "dosyntax - checking tr ($key)\n" if $opt_V; 
-
 	return $O_OK;
     }
     
@@ -2272,8 +2265,6 @@ sub dosyntax {
 #   
     if ($key eq "ue") {
        
-       print STDERR "dosyntax - checking ue ($key)\n" if $opt_V; 
-       
 	return $O_OK;
     }
 #
@@ -2281,8 +2272,6 @@ sub dosyntax {
 #    
     if ($key eq "uf") {
        
-       print STDERR "dosyntax - checking uf ($key)\n" if $opt_V; 
-       
 	return $O_OK;
     }
 #
@@ -2291,8 +2280,6 @@ sub dosyntax {
     
     if ($key eq "ui") {
        
-       print STDERR "dosyntax - checking ui ($key)\n" if $opt_V; 
-       
 	return $O_OK;
     }
 #
@@ -2300,15 +2287,16 @@ sub dosyntax {
 #
     if ($key eq "uw") {
     
-       print STDERR "dosyntax - checking uw ($key)\n" if $opt_V; 
-       
 	return $O_OK;
     }
     
-    return $O_ERROR, "Internal error: $key is not syntax checked" if ($ATTR{$key});
-    
-    return $O_ERROR, "Internal error: unknown attribute found during syntax check ($key)";
-    
+    if ($ATTR{$key}) {
+        return $O_ERROR, "Internal error: $key is not syntax checked" 
+	    unless grep(/\b$key\b/, $GEN{$type});
+    } else {
+      return $O_ERROR, 
+	  "Internal error: unknown attribute found during syntax check ($key)";
+    }
 }
 
 1;
